/*
 Copyright 2016 Google Inc. All Rights Reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

describe(`cache-revisioned-e2e.js`, function() {
  const deleteIndexedDB = () => {
    return new Promise((resolve, reject) => {
      // TODO: Move to constants
      const req = indexedDB.deleteDatabase('workbox-precaching');
      req.onsuccess = function() {
        resolve();
      };
      req.onerror = function() {
        reject();
      };
      req.onblocked = function() {
        console.error('Database deletion is blocked.');
      };
    });
  };

  beforeEach(function() {
    return goog.swUtils.cleanState().then(deleteIndexedDB);
  });

  afterEach(function() {
    return goog.swUtils.cleanState().then(deleteIndexedDB);
  });

  const testCacheEntries = (fileSet) => {
    return window.caches.keys()
    .then((cacheNames) => {
      cacheNames.length.should.equal(1);
      return window.caches.open(cacheNames[0]);
    })
    .then((cache) => {
      return cache.keys();
    })
    .then((cachedResponses) => {
      cachedResponses.length.should.equal(fileSet.length);

      fileSet.forEach((assetAndHash) => {
        let matchingResponse = null;
        cachedResponses.forEach((cachedResponse) => {
          let desiredPath = assetAndHash;
          if (typeof assetAndHash !== 'string') {
            desiredPath = assetAndHash.url;
          }

          if (cachedResponse.url.indexOf(desiredPath) !== -1) {
            matchingResponse = cachedResponse;
            return;
          }
        });

        expect(matchingResponse).to.exist;
      });
    });
  };

  const testFileSet = (iframe, swPath, fileSet) => {
    return testCacheEntries(fileSet)
    .then(() => {
      let responses = {};
      const promises = fileSet.map((assetAndHash) => {
        let url = assetAndHash;
        if (typeof assetAndHash === 'object') {
          url = assetAndHash.url;
        }

        // This handles relative URL's that will be relative to the service
        // works path.
        const parsedURL = new URL(
          url,
          new URL(swPath, location).toString()
        ).toString();
        return iframe.contentWindow.fetch(parsedURL)
        .then((cachedResponse) => {
          if (cachedResponse.type === 'opaque') {
            responses[url] = null;
          } else {
            return cachedResponse.text()
            .then((bodyText) => {
              responses[url] = bodyText;
            });
          }
        });
      });
      return Promise.all(promises)
      .then(() => {
        return responses;
      });
    });
  };

  const compareCachedAssets = function(beforeData, afterData) {
    afterData.cacheList.forEach((afterAssetAndHash) => {
      if (typeof afterAssetAndHash === 'string') {
        afterAssetAndHash = {url: afterAssetAndHash, revision: afterAssetAndHash};
      }

      let matchingBeforeAssetAndHash = null;
      beforeData.cacheList.forEach((beforeAssetAndHash) => {
        if (typeof beforeAssetAndHash === 'string') {
          beforeAssetAndHash = {url: beforeAssetAndHash, revision: beforeAssetAndHash};
        }

        if (beforeAssetAndHash.url === afterAssetAndHash.url) {
          matchingBeforeAssetAndHash = beforeAssetAndHash;
        }
      });

      if (!matchingBeforeAssetAndHash) {
        return;
      }

      const beforeResponseBody = beforeData.cachedResponses[afterAssetAndHash.url];
      const afterResponseBody = afterData.cachedResponses[afterAssetAndHash.url];

      if (matchingBeforeAssetAndHash.revision === afterAssetAndHash.revision) {
        // The request should be the same
        beforeResponseBody.should.equal(afterResponseBody);
      } else {
        // The request should be different
        beforeResponseBody.should.not.equal(afterResponseBody);
      }
    });
  };

  it(`should cache and fetch files`, function() {
    const testSet = workbox.__TEST_DATA['workbox-sw']['revisioned'];
    const allAssets1 = testSet['set-1']
      .concat(testSet['set-2'])
      .concat(testSet['set-3']);

    const allAssets2 = testSet['set-4']
      .concat(testSet['set-5'])
      .concat(testSet['set-6']);

    const sw1 = '/packages/workbox-sw/test/static/sw/cache-revisioned-1.js';
    const sw2 = '/packages/workbox-sw/test/static/sw/cache-revisioned-2.js';
    return goog.swUtils.activateSW(sw1).then((iframe) => {
      return testFileSet(iframe, sw1, allAssets1);
    })
    .then((step1Responses) => {
      return goog.swUtils.activateSW(sw2).then((iframe) => {
        return testFileSet(iframe, sw2, allAssets2);
      })
      .then((step2Responses) => {
        compareCachedAssets({
          cacheList: allAssets1,
          cachedResponses: step1Responses,
        }, {
          cacheList: allAssets2,
          cachedResponses: step2Responses,
        });
      });
    });
  });
});

/*
 Copyright 2016 Google Inc. All Rights Reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

describe(`Test ignore url params matching`, function() {
  it(`should match with correct default params`, function() {
    return goog.swUtils.activateSW('../static/sw/ignore-url-params.js')
    .then((iframe) => {
      let cacheEntry = null;
      return iframe.contentWindow.caches.match('/__echo/date/hello')
      .then((response) => {
        return response.text();
      })
      .then((cE) => {
        // Just make sure we cached something from the test server.
        cE.indexOf('hello').should.not.equal(-1);
        cacheEntry = cE;
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello');
      })
      .then((result) => {
        return result.text();
      })
      .then((response) => {
        response.should.equal(cacheEntry);
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?utm_=something');
      })
      .then((result) => {
        return result.text();
      })
      .then((response) => {
        response.should.equal(cacheEntry);
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?utm_=something&should=fail')
        .then(() => {
          throw new Error('Expected this to error');
        }, () => {});
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?should=fail&utm_=something')
        .then(() => {
          throw new Error('Expected this to error');
        }, () => {});
      });
    });
  });

  it(`should match with custom params`, function() {
    return goog.swUtils.activateSW('../static/sw/ignore-url-params-custom.js')
    .then((iframe) => {
      let cacheEntry = null;
      return iframe.contentWindow.caches.match('/__echo/date/hello')
      .then((response) => {
        return response.text();
      })
      .then((cE) => {
        // Just make sure we cached something from the test server.
        cE.indexOf('hello').should.not.equal(-1);
        cacheEntry = cE;
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello');
      })
      .then((result) => {
        return result.text();
      })
      .then((response) => {
        response.should.equal(cacheEntry);
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?example=something')
        .then((result) => {
          return result.text();
        })
        .then((response) => {
          response.should.equal(cacheEntry);
        });
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?somethingtestsomething=something')
        .then((result) => {
          return result.text();
        })
        .then((response) => {
          response.should.equal(cacheEntry);
        });
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?somethingtestsomething=something&example=testing')
        .then((result) => {
          return result.text();
        })
        .then((response) => {
          response.should.equal(cacheEntry);
        });
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?somethingtestsomething=something&example=testing&utm_=something')
        .then(() => {
          throw new Error('Expected this to error');
        }, () => {});
      })
      .then(() => {
        return iframe.contentWindow.fetch('/__echo/date/hello?utm_=something')
        .then(() => {
          throw new Error('Expected this to error');
        }, () => {});
      });
    });
  });

  it(`should ignore the URL's hash when routing to a precached URL`, async function() {
    const responseBody = 'hello';
    const urlObject = new URL(`/__echo/date/${responseBody}`, location);
    urlObject.hash = 'should-be-ignored';

    const iframe = await goog.swUtils.activateSW('../static/sw/ignore-url-params.js');
    const fetchResponse = await iframe.contentWindow.fetch(urlObject.href);
    const fetchResponseBody = await fetchResponse.text();

    expect(fetchResponseBody.startsWith(responseBody)).to.be.true;
  });
});

/*
 Copyright 2016 Google Inc. All Rights Reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

describe(`Test Behaviors of Loading the Script`, function() {
  this.timeout(5 * 60 * 1000);

  const deleteIndexedDB = () => {
    return new Promise((resolve, reject) => {
      // TODO: Move to constants
      const req = indexedDB.deleteDatabase('workbox-precaching');
      req.onsuccess = function() {
        resolve();
      };
      req.onerror = function() {
        reject();
      };
      req.onblocked = function() {
        console.error('Database deletion is blocked.');
      };
    });
  };

  beforeEach(function() {
    return goog.swUtils.cleanState().then(deleteIndexedDB);
  });

  afterEach(function() {
    return goog.swUtils.cleanState().then(deleteIndexedDB);
  });

  it(`should print an error when added to the window.`, function() {
    this.timeout(2000);

    return new Promise((resolve, reject) => {
      window.onerror = (msg, url, lineNo, columnNo, error) => {
        window.onerror = null;

        if (error.name === 'not-in-sw') {
          resolve();
          return true;
        } else {
          reject('Unexpected error received.');
          return false;
        }
      };

      const scriptElement = document.createElement('script');
      scriptElement.src = '/__test/bundle/workbox-sw';
      scriptElement.addEventListener('error', (event) => {
        reject();
      });
      document.head.appendChild(scriptElement);
    });
  });
});

/*
 Copyright 2016 Google Inc. All Rights Reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

describe(`End to End Test of registerNavigationRoute()`, function() {
  // This uses {once: true} to ensure that each load event handler is only called once.
  // It also modifies the iframe.src instead of using fetch(), since you can't use fetch()
  // to make a request with mode === 'navigate'.
  it(`should serve the App Shell content based on the whitelist/blacklist`, function(callback) {
    goog.swUtils.activateSW('../static/sw/navigation-route.js')
      .then((iframe) => {
        iframe.addEventListener('load', () => {
          expect(iframe.contentWindow.document.body.innerText).to.equal('navigation');

          iframe.addEventListener('load', () => {
            expect(iframe.contentWindow.document.body.innerText).not.to.equal('navigation');
            callback();
          }, {once: true});

          iframe.src = iframe.src + '/blacklisted';
        }, {once: true});

        iframe.src = iframe.src + '/navigation';
      });
  });
});
//# sourceMappingURL=browser-bundle.js.map
